@@ -124,7 +124,11 @@ class TDMConsole(code.InteractiveConsole):
             """
             src_p = robot_code()
             # compile, load, run, and set scratchpad without checking the result
-            self.run_program(src_p, language="python", wait=wait)
+            try:
+                self.run_program(src_p, language="python", wait=wait)
+            except KeyboardInterrupt:
+                # avoid long exception message with stack trace
+                print("Interrupted")
 
         def stop():
             """Stop the program running on the robot.
@@ -277,6 +281,9 @@ class TDMConsole(code.InteractiveConsole):
 
     def reset_sync_var(self):
         self.sync_var = self.sync_var_vm.copy()
+        # also pick user variables not known at connection time
+        for name in self.node.var:
+            self.sync_var.add(name)
 
     def process_events(self, on_event_data=None):
         """Listen to events sent by the program running on the robot and process
@@ -462,6 +469,12 @@ class TDMConsole(code.InteractiveConsole):
         def do_node(node):
             nonlocal globals, var_got, var_set, var_global
 
+            # alternative to direct use of isinstance for node types
+            # which aren't defined in Python 3.6
+            def isinst(obj, ast_name):
+                return (hasattr(ast, ast_name)
+                        and isinstance(obj, getattr(ast, ast_name)))
+
             if isinstance(node, ast.Assign):
                 for target in node.targets:
                     if isinstance(target, ast.Tuple):
@@ -481,8 +494,8 @@ class TDMConsole(code.InteractiveConsole):
             elif isinstance(node, ast.BoolOp):
                 do_nodes(node.values)
             elif isinstance(node, ast.Call):
+                do_node(node.func)
                 do_nodes(node.args)
-                ast.dump(node)
                 if isinstance(node.func, ast.Name):
                     fun_name = node.func.id
                     if (fun_name not in ATranspiler.PREDEFINED_FUNCTIONS
@@ -539,11 +552,21 @@ class TDMConsole(code.InteractiveConsole):
                 do_node(node.value)
             elif isinstance(node, ast.JoinedStr):
                 do_nodes(node.values)
+            elif isinstance(node, ast.Lambda):
+                do_node(node.body)
             elif isinstance(node, ast.List):
                 do_nodes(node.elts)
             elif isinstance(node, ast.ListComp):
                 do_node(node.elt)
                 do_nodes(node.generators)
+            elif isinst(node, "Match"):
+                do_node(node.subject)
+                for c in node.cases:
+                    do_node(c.pattern)
+                    do_node(c.guard)
+                    do_nodes(c.body)
+            elif isinst(node, "MatchValue"):
+                do_node(node.value)
             elif isinstance(node, ast.Name):
                 if node.id in globals:
                     var_got.add(node.id)
@@ -586,11 +609,12 @@ class TDMConsole(code.InteractiveConsole):
                             (ast.AsyncFunctionDef, ast.Attribute,
                              ast.Break,
                              ast.ClassDef, ast.Constant, ast.Continue,
-                             ast.Delete, ast.FunctionDef,
+                             ast.Delete,
+                             ast.FunctionDef,
                              ast.Import, ast.ImportFrom,
-                             ast.Lambda,
                              ast.NameConstant, ast.Nonlocal, ast.Num,
-                             ast.Pass)):
+                             ast.Pass,
+                             ast.Str)):
                 pass
             elif node is not None:
                 print("Unchecked", ast.dump(node))
@@ -631,7 +655,7 @@ class TDMConsole(code.InteractiveConsole):
             # print("pre_run error", e)
             pass
 
-    def get_function_def_src(self, node):
+    def get_function_def_src(self, node, next_node):
         # use bytes b/c col_offset is in bytes
         src_b = bytes(self.cmd_src, "utf-8")
 
@@ -653,14 +677,10 @@ class TDMConsole(code.InteractiveConsole):
                 index_from -= 1
             if src_b[index_from - 1] == ord("@"):
                 index_from -= 1
-            else:
-                raise SyntaxError("Internal error",
-                                  ("<stdin>",
-                                   node.decorator_list[0].lineno,
-                                   node.decorator_list[0].col_offset,
-                                   self.cmd_src))
 
-        index_to = index(node.end_lineno, node.end_col_offset)
+        index_to = (index(next_node.lineno, next_node.col_offset)
+                    if next_node is not None
+                    else len(src_b))
 
         return str(src_b[index_from : index_to], "utf-8") + "\n"
 
@@ -672,13 +692,16 @@ class TDMConsole(code.InteractiveConsole):
         try:
             if (self.cmd_tree is not None and
                 self.cmd_tree.body is not None):
-                for statement in self.cmd_tree.body:
+                for i, statement in enumerate(self.cmd_tree.body):
                     if isinstance(statement, ast.FunctionDef):
                         # keep function source code
                         var_got, var_set, var_gl, fun_called = self.find_global_var(statement.body,
                                                                                     globals=set())
                         self.fun_defs[statement.name] = {
-                            "src": self.get_function_def_src(statement),
+                            "src": self.get_function_def_src(statement,
+                                                             self.cmd_tree.body[i + 1]
+                                                             if i + 1 < len(self.cmd_tree.body)
+                                                             else None),
                             "in": var_got,
                             "out": var_set,
                             "global": var_gl,
